<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use \Util\Exception;

class FiscalFunctions {

	protected $fiscalFunctionsService;
	protected $version;
	
	public function __construct(\Service\FiscalFunctions $fiscalFunctionsService){
		$this->fiscalFunctionsService = $fiscalFunctionsService;
		$this->version = $this->getVersion();
	}

	const NAME_APP = 'odhenPOS';

	public function vendasCPFCNPJ(Request\Filter $request, Response $response) {
		try {
			$params = $request->getFilterCriteria()->getConditions();			
			
			$cdFilial = $params[0]['value'];
			$cpfCnpj = $params[1]['value'];
			$periodo = $params[2]['value'];
			$result = $this->fiscalFunctionsService->vendasCPFCNPJ($cdFilial, $cpfCnpj, $periodo, self::NAME_APP, $this->version);

		} catch (\Exception $e) {
			Exception::logException($e);
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vendasCPFCNPJ', $result));
	}

	public function registroPaf(Request\Filter $request, Response $response) {
		try {
			$params = $request->getFilterCriteria()->getConditions();			
			
			$cdFilial = $params[0]['value'];
			$cdCaixa = $params[1]['value'];
			$dataInicio = $params[2]['value'];
			$dataFim = $params[3]['value'];

			$result = $this->fiscalFunctionsService->registroPaf($cdFilial, $cdCaixa, $dataInicio, $dataFim, self::NAME_APP);

		} catch (\Exception $e) {
			Exception::logException($e);
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('registroPaf', $result));
	}

	public function requisicaoExterna(Request\Filter $request, Response $response) {
		try {
			$params = $request->getFilterCriteria()->getConditions();			
			
			$cdFilial = $params[0]['value'];
			$dataInicio = $params[1]['value'];
			$dataFim = $params[2]['value'];

			$result = $this->fiscalFunctionsService->requisicaoExterna($cdFilial, $dataInicio, $dataFim, self::NAME_APP, $this->version);

		} catch (\Exception $e) {
			Exception::logException($e);
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('requisicaoExterna', $result));
	}

	public function getVersion() {
		$versionFile = json_decode(file_get_contents(str_replace('\\', DIRECTORY_SEPARATOR, __DIR__. '/../../version.json')), true);
		return $versionFile['backVersion'];
	}

}